VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CNotifier"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' ------------------------------------------------------------------------
'    Copyright  1999 Willamette Industries, Inc.  All rights reserved.
'
' You have a royalty-free right to use, modify, reproduce and distribute
' this source code (and/or any modified version) in any way you find
' useful, provided that you agree that Willamette Industries has no
' warranty, obligations or liability for any use of this code.
'
' Written April 15, 1999 by Darryl Luther
' ------------------------------------------------------------------------
Option Explicit

' API Constants
Private Const ERROR_PATH_NOT_FOUND = 3&
Private Const INVALID_HANDLE_VALUE = -1
Private Const FILE_NOTIFY_CHANGE_ATTRIBUTES = &H4
Private Const FILE_NOTIFY_CHANGE_DIR_NAME = &H2
Private Const FILE_NOTIFY_CHANGE_FILE_NAME = &H1
Private Const FILE_NOTIFY_CHANGE_LAST_WRITE = &H10
Private Const FILE_NOTIFY_CHANGE_SECURITY = &H100
Private Const FILE_NOTIFY_CHANGE_SIZE = &H8
Private Const FILE_NOTIFY_CHANGE_ALL = FILE_NOTIFY_CHANGE_ATTRIBUTES + FILE_NOTIFY_CHANGE_DIR_NAME + _
                                       FILE_NOTIFY_CHANGE_FILE_NAME + FILE_NOTIFY_CHANGE_LAST_WRITE + _
                                       FILE_NOTIFY_CHANGE_SECURITY + FILE_NOTIFY_CHANGE_SIZE
Private Const WAIT_OBJECT_0 = &H0

' API Functions
Private Declare Function FindFirstChangeNotification Lib "kernel32" _
  Alias "FindFirstChangeNotificationA" _
    ( _
      ByVal lpPathName As String, _
      ByVal bWatchSubtree As Long, _
      ByVal dwNotifyFilter As Long _
    ) _
  As Long
  
Private Declare Function FindNextChangeNotification Lib "kernel32" _
    ( _
      ByVal hChangeHandle As Long _
    ) _
  As Long
  
Private Declare Function FindCloseChangeNotification Lib "kernel32" _
    ( _
      ByVal hChangeHandle As Long _
    ) _
  As Long
  
Private Declare Function WaitForSingleObject Lib "kernel32" _
    ( _
      ByVal hHandle As Long, _
      ByVal dwMilliseconds As Long _
    ) _
  As Long

' Rename the FILE_NOTIFY_CHANGE_* constants to more user friendly names
Public Enum eFilters
  efChangeInAttributes = FILE_NOTIFY_CHANGE_ATTRIBUTES
  efChangeInDirectoryName = FILE_NOTIFY_CHANGE_DIR_NAME
  efChangeInFileName = FILE_NOTIFY_CHANGE_FILE_NAME
  efChangeInLastWrite = FILE_NOTIFY_CHANGE_LAST_WRITE
  efChangeInSecurity = FILE_NOTIFY_CHANGE_SECURITY
  efChangeInSize = FILE_NOTIFY_CHANGE_SIZE
  efChangeInAny = FILE_NOTIFY_CHANGE_ALL
End Enum

' Module level variables
Private mhChange As Long        ' Handle to the change notification object
Private miFreq As Long          ' Time (in milliseconds) to wait for change in directory

' Events to raise back to client
Public Event ChangeOccurred()   ' Event to raise when a filtered change has occurred
Attribute ChangeOccurred.VB_Description = "This event is raised whenever one of the filtered change events occurs."
Public Event NotifierFailing(ReturnCode As Long)    ' Event to raise when notifier fails
Attribute NotifierFailing.VB_Description = "This event is not currently implemented, but will be used in the future to indicate an internal error occurred while setting up or waiting for a change notification object."

'---------------------------------------------------------------------------------------
' Function:   StartNotify
'
' Purpose:      Called from client to start the change notification process.
' Inputs:       PathToWatch - string containing the directory path for the change
'                             notification to watch.
'               CheckFrequency - long containing the number of milliseconds to wait
'                                for the change notification object.  This parameter
'                                is optional and defaults to 5000 (5 seconds).
'               ChangeFilters - long (enum) containing any combination of the
'                               filters specified in the eFilters enum.  No bounds
'                               checking is performed on these flags.  This parameter
'                               is optional and defaults to efChangeInFileName (this
'                               includes adding of new files to the directory).
'               ProcessSubdiretories - boolean flag indicating whether all
'                                      subdirectories should be watched (True) or just
'                                      the directory passed in PathToWatch (False).
'                                      This parameter is optional and defaults to True.
' Outputs:      Long.  0 for success, ERROR_ constant otherwise.
' Assumptions:  ChangeFilters is a valid value
'---------------------------------------------------------------------------------------
Public Function StartNotify(PathToWatch As String, Optional CheckFrequency As Long = 5000, _
                            Optional ChangeFilters As eFilters = efChangeInFileName, _
                            Optional ProcessSubdirectories As Boolean = True) As Long
Attribute StartNotify.VB_Description = "Call this method to begin the change notification process."
  Dim sPath As String
  ' Ensure a path is passed
  If (Trim$(PathToWatch) = "") Then
    StartNotify = ERROR_PATH_NOT_FOUND
  Else
    ' Format path variable...
    sPath = Trim$(PathToWatch)
    ' ...get the handle to the change notification object...
    ' (And[ing] ProcessSubdirectories with one will ensure either a 0 or a 1.  VB's True
    ' is -1 which doesn't play nice with the API.)
    mhChange = FindFirstChangeNotification(sPath, (ProcessSubdirectories And 1), ChangeFilters)
    ' ...check for an invalid handle...
    If (mhChange = INVALID_HANDLE_VALUE) Then
      StartNotify = Err.LastDllError
    Else
      ' ...save the frequency with which to check the directory for changes...
      miFreq = CheckFrequency
      ' ...and fire up the timer to start this whole process (give it a second so
      ' control is returned to the client app).
      Call StartTimer(Me, 1000)
      StartNotify = 0
    End If
  End If
End Function

'---------------------------------------------------------------------------------------
' Function:     EndNotify
'
' Purpose:      Called from client to end the change notification process.
' Inputs:       None.
' Outputs:      Long.  0 for success, ERROR_ constant otherwise.
' Assumptions:  None.
'---------------------------------------------------------------------------------------
Public Function EndNotify() As Long
Attribute EndNotify.VB_Description = "Call this method to stop the notify process.   This should be called prior to setting the object reference to nothing.  Depending on how frequently the notification is run it may take a few moments for this to return."
  Dim iRet As Long
  ' Close the change notification object
  iRet = FindCloseChangeNotification(mhChange)
  ' Set the handle variable to 0 so the process will end
  mhChange = 0
  ' Assign the success or failure of the above call to the function return value.  This
  ' could be more robust to return the actual reason for failure.
  If iRet = 0 Then
    EndNotify = Err.LastDllError
  Else
    EndNotify = 0
  End If
End Function

'---------------------------------------------------------------------------------------
' Subroutine:   KickOff
'
' Purpose:      Called from MTimer module to begin and run the process of watching the
'               directory.
' Inputs:       None.
' Outputs:      N/A.
' Assumptions:  None.
'---------------------------------------------------------------------------------------
Friend Sub KickOff()
  Dim iChangeObject As Long
  ' This loop will continue until EndNotify has been called
  Do While Not (mhChange = 0)
    ' Wait for a change in directory
    iChangeObject = WaitForSingleObject(mhChange, miFreq)
    ' If an actual change occurred...
    If iChangeObject = WAIT_OBJECT_0 Then
      ' ...raise an event to the clien.
      RaiseEvent ChangeOccurred
      ' Reset the change notification object
      Call FindNextChangeNotification(mhChange)
    End If
    ' Without this DoEvents, the loop is too tight to allow EndNotify to be called.
    DoEvents
  Loop
End Sub
